@@ -389,10 +389,12 @@ public class BundleLoader implements ClassLoaderDelegate {
 			result = findLocalClass(name);
 		if (result != null)
 			return result;
-		// 6) attempt to find a dynamic import source
-		source = findDynamicSource(pkgName);
-		if (source != null)
-			result = source.loadClass(name);
+		// 6) attempt to find a dynamic import source; only do this if a required source was not found
+		if (source == null) {
+			source = findDynamicSource(pkgName);
+			if (source != null)
+				result = source.loadClass(name);
+		}
 		// do buddy policy loading
 		if (result == null && policy != null)
 			result = policy.doBuddyClassLoading(name);
@@ -482,10 +484,12 @@ public class BundleLoader implements ClassLoaderDelegate {
 			result = findLocalResource(name);
 		if (result != null)
 			return result;
-		// 6) attempt to find a dynamic import source
-		source = findDynamicSource(pkgName);
-		if (source != null)
-			result = source.getResource(name);
+		// 6) attempt to find a dynamic import source; only do this if a required source was not found
+		if (source == null) {
+			source = findDynamicSource(pkgName);
+			if (source != null)
+				result = source.getResource(name);
+		}
 		// do buddy policy loading
 		if (result == null && policy != null)
 			return policy.doBuddyResourceLoading(name);
@@ -532,12 +536,26 @@ public class BundleLoader implements ClassLoaderDelegate {
 		// 5) search the local bundle
 		if (result == null)
 			result = findLocalResources(name);
+		else {
+			//compound the required source results with the local ones
+			Enumeration localResults = findLocalResources(name);
+			if (localResults != null) {
+				Vector compoundResults = new Vector();
+				while (result.hasMoreElements())
+					compoundResults.add(result.nextElement());
+				while (localResults.hasMoreElements())
+					compoundResults.add(localResults.nextElement());
+				result = compoundResults.elements();
+			}
+		}
 		if (result != null)
 			return result;
-		// 6) attempt to find a dynamic import source
-		source = findDynamicSource(pkgName);
-		if (source != null)
-			result = source.getResources(name);
+		// 6) attempt to find a dynamic import source; only do this if a required source was not found
+		if (source == null) {
+			source = findDynamicSource(pkgName);
+			if (source != null)
+				result = source.getResources(name);
+		}
 		if (result == null && policy != null)
 			result = policy.doBuddyResourcesLoading(name);
 		return result;
@@ -830,10 +848,8 @@ public class BundleLoader implements ClassLoaderDelegate {
 		PackageSource result = findImportedSource(pkgName);
 		if (result != null)
 			return result;
-		result = findRequiredSource(pkgName);
-		if (result == null)
-			result = findDynamicSource(pkgName);
-		return result;
+		// Note that dynamic imports are not checked to avoid aggressive wiring (bug 105779)  
+		return findRequiredSource(pkgName);
 	}
 
 	private PackageSource findImportedSource(String pkgName) {
