@@ -44,6 +44,8 @@ import org.eclipse.ui.internal.presentations.PresentablePart;
 import org.eclipse.ui.internal.registry.EditorDescriptor;
 import org.eclipse.ui.internal.util.Util;
 import org.eclipse.ui.part.IWorkbenchPartOrientation;
+import org.eclipse.ui.part.MultiEditor;
+import org.eclipse.ui.part.MultiEditorInput;
 import org.eclipse.ui.presentations.IPresentablePart;
 
 public class EditorReference extends WorkbenchPartReference implements
@@ -80,6 +82,13 @@ public class EditorReference extends WorkbenchPartReference implements
 
     IEditorInput restoredInput;
     
+	/**
+	 * If the reference is instantiated as a MultiEditor, we need to dispose the
+	 * inner references correctly.
+	 */
+	private IEditorReference[] multiEditorChildren = null;
+
+    
     public EditorReference(EditorManager manager, IEditorInput input, EditorDescriptor desc) {
         this.manager = manager;
         initListenersAndHandlers();
@@ -241,7 +250,17 @@ public class EditorReference extends WorkbenchPartReference implements
     }
 
     protected void doDisposePart() {
-        if (part != null) {
+		if (multiEditorChildren != null) {
+			for (int i = 0; i < multiEditorChildren.length; ++i) {
+				EditorReference ref = (EditorReference) multiEditorChildren[i];
+				if (ref != null) {
+					ref.dispose();
+				}
+			}
+			multiEditorChildren = null;
+		}
+
+    	if (part != null) {
             EditorSite site = (EditorSite) ((IEditorPart)part).getEditorSite();
             manager.disposeEditorActionBars((EditorActionBars) site.getActionBars());
             site.dispose();
@@ -534,7 +553,7 @@ public class EditorReference extends WorkbenchPartReference implements
                 throw new PartInitException(NLS.bind(WorkbenchMessages.EditorManager_missing_editor_descriptor, editorID)); //$NON-NLS-1$
             }
             
-            IEditorPart part;
+            IEditorPart part = null;
             
             if (desc.isInternal()) {    
                 // Create an editor instance.
@@ -545,6 +564,11 @@ public class EditorReference extends WorkbenchPartReference implements
                     UIStats.end(UIStats.CREATE_PART, this, editorID);
                 }
                 
+                if (part != null && part instanceof MultiEditor) {
+					multiEditorChildren = manager.openMultiEditor(this,
+						(MultiEditor) part, (MultiEditorInput) editorInput);
+				}
+                
             } else if (desc.getId().equals(
                     IEditorRegistry.SYSTEM_INPLACE_EDITOR_ID)) {
                 
@@ -635,4 +659,16 @@ public class EditorReference extends WorkbenchPartReference implements
     
     }
     
-}
\ No newline at end of file
+    /**
+	 * A quick way of finding out if this reference points to a MultiEditor.
+	 * It depends on the fact that a MultiEditor does not lazily 
+	 * instantiate it's child editors.
+	 * 
+	 * @return true if it has inner editor reference or the input is
+	 * MultiEditorInput.
+	 */
+	public boolean isMultiReference() {
+		return multiEditorChildren != null
+				|| restoredInput instanceof MultiEditorInput;
+	}
+}
